<?php
	class YouBazePreorderVIES
	{
		private $apiurl = 'http://ec.europa.eu/taxation_customs/vies/viesquer.do';
		public static $countries = array('AT', 'BE', 'BG', 'CY', 'CZ', 'DE', 'DK', 'EE', 'ES', 'FI', 'FR', 'GB', 'GR', 'HU', 'IE', 'IT', 'LT', 'LU', 'LV', 'MT', 'NL', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK');
		
		public function __construct()
		{
			
		}
		
		public function validate($countryCode, $vatnr)
		{
			if (!in_array($countryCode, self::$countries)) return false;
			
			$vatnr = self::stripVATNumber($vatnr);
			if ($vatnr=='') return false;
			
			$result = $this->postToVIES(array(
				'ms' => $this->getVATNumberPrefix($countryCode), 
				'iso' => $this->getVATNumberPrefix($countryCode),
				'vat'=>$vatnr
			));
			
			//var_dump($result);
			
			if ($result===false) return null;
			
			return strpos($result, 'Yes, valid VAT number')!==false;
		}
		
		public static function formatVATNumber($vatnr)
		{
			return strtoupper(str_replace(array(' ', '-', '_', '.'), '', $vatnr));
		}
		
		public static function stripVATNumber($vatnr)
		{
			$vatnr = self::formatVATNumber($vatnr);
			$possibleCC = substr($vatnr, 0, 2);
			if (in_array($possibleCC, array_merge(self::$countries, array('EL'))))
				$vatnr = substr($vatnr, 2);
			return $vatnr;
		}
		
		public static function isEUMemberState($countryCode)
		{
			return in_array($countryCode, self::$countries);
		}
		
		private function getVATNumberPrefix($countryCode)
		{
			return $countryCode=='GR' ? 'EL' : $countryCode;
		}
		
		private function postToVIES($postvars = array())
		{
			$url = parse_url($this->apiurl);
			
			$fp = fsockopen(($url['scheme']=='http' ? '' : 'ssl://').$url['host'], $url['scheme']=='http' ? 80 : 443, $errno, $errstr, 5);
			
			if (!$fp)
			{
				return false;

			} else 
			{
				$content = http_build_query($postvars);
				$path = (isset($url['path']) ? $url['path'] : '') . (isset($url['query']) ? '?'.$url['query'] : '');
				$out = "POST ".$path." HTTP/1.0\r\n";
				$out .= "Host: ".$url['host']."\r\n";
				$out .= "Accept: text/*\r\n";
				$out .= "User-Agent: Firefox\r\n";
				$out .= "Accept-Language: en\r\n";
				$out .= "Content-Type: application/x-www-form-urlencoded\r\n";
				$out .= "Content-Length: ".strlen($content)."\r\n";
				$out .= "Connection: Close\r\n\r\n";
				$out .= $content;
				
				//die($out);
				
				fwrite($fp, $out);
				$return = '';
				while (!feof($fp))
				{
					$return .= fgets($fp);
				}
				fclose($fp);
				
				//die($return);
				
				$parts = explode("\r\n\r\n", $return);
				if (count($parts)>1)
				{
					unset($parts[0]);			
					return implode("\r\n\r\n", $parts);
				}

				return false;
			}
		}
	}
?>