<?php
	/*
	Plugin Name: YouBaze Pre-order formulier
	Description: Integreer pre-order bestelformulieren in je WordPress pagina en importeer de bestellingen met 1 klik in je YouBaze-account (YouBaze is gratis cloudsoftware voor het MKB).
	Version: 4.3.1
	Author: YouBaze NL
	Author URI: https://www.youbaze.com/nl/
	*/
	
	include "class.YouBazePreorderVIES.php";
	register_activation_hook(__FILE__, 'youbaze_preorder_activate');
	add_action('admin_menu', 'youbaze_preorder_menu');
		
	function youbaze_preorder_getOptionName()
	{
		return 'youbaze_preorder';
	}
		
	function youbaze_preorder_activate()
	{
		$optionexists = false;
		$vals = array();
		$currentoption = get_option(youbaze_preorder_getOptionName());
		if ($currentoption!==false)
		{
			$vals = $currentoption;
			$optionexists = true;
		}
		
		$defaultvals = array('youbaze_email' => 'your.youbaze.email@example.com');
		foreach ($defaultvals as $k => $defaultval)
		{
			if (!isset($vals[$k]))
				$vals[$k] = $defaultval;
		}
		
		if ($optionexists)
			update_option(youbaze_preorder_getOptionName(), $vals);
		else
			add_option(youbaze_preorder_getOptionName(), $vals);
	}
	
	function youbaze_preorder_menu() 
	{
		add_options_page('YouBazePreorder', 'YouBazePreorder', 'manage_options', 'youbaze_preorder', 'youbaze_preorder_options');
	}

	function youbaze_preorder_options() 
	{
		if (!current_user_can('manage_options'))
			wp_die('Je hebt geen toestemming om deze pagina te bewerken.');
		
		$saved = false;
		$error = false;
		$fields = array('youbaze_email');
		$optionvalue = get_option(youbaze_preorder_getOptionName());
		
		foreach ($fields as $field)
		{
			$$field = isset($_POST[$field]) ? trim($_POST[$field]) : $optionvalue[$field];
		}

		if (isset($_POST['Submit']))
		{
			if (!youbaze_preorder_validateEmail($youbaze_email))
				$error = 'Het e-mailadres is niet juist.';

			if ($error===false)
			{
				$vals = array();
				foreach ($optionvalue as $key => $value)
					$vals[$key] = $$key;

				update_option(youbaze_preorder_getOptionName(), $vals);
				$optionvalue = $vals;
				$saved = true;
			}
		}
		?>
		<div class="wrap">
			<h2>YouBaze Preorder Forms</h2>
			<?php
				if ($saved)
				{
					?>
					<div class="updated"><p><strong>Your settings have been saved</strong></p></div>
					<?php
				} elseif ($error!==false)
				{
					?>
					<div class="error"><p><strong><?php echo youbaze_preorder_html($error);?></strong></p></div>
					<?php
				}
			?>
			
			<form id="youbazeform" name="form1" method="post" action="" enctype="multipart/form-data">
				<h3>General settings</h3>
				<table class="form-table">
					<tbody>
						<tr>
							<td><label for="youbaze_email">YouBaze email address</label></td>
							<td><input type="text" name="youbaze_email" id="youbaze_email" value="<?php echo youbaze_preorder_html($youbaze_email);?>" /></td>
						</tr>
					</tbody>
				</table>
				<p>Set up your preorder form in a widget and add this widget to the page you like.</p>
				<p class="submit">
					<input type="submit" name="Submit" class="button-primary" value="Opslaan" />
				</p>
			</form>
		</div>
		<?php
	}
	
	function youbaze_preorder_validateEmail($email)
	{
		/*return preg_match('/^(?!(?>(?1)"?(?>\\\[ -~]|[^"])"?(?1)){255,})(?!(?>(?1)"?(?>\\\[ -~]|[^"])"?(?1)){65,}@)((?>(?>(?>((?>(?>(?>\x0D\x0A)?[\t ])+|(?>[\t ]*\x0D\x0A)?[\t ]+)?)(\((?>(?2)(?>[\x01-\x08\x0B\x0C\x0E-\'*-\[\]-\x7F]|\\\[\x00-\x7F]|(?3)))*(?2)\)))+(?2))|(?2))?)([!#-\'*+\/-9=?^-~-]+|"(?>(?2)(?>[\x01-\x08\x0B\x0C\x0E-!#-\[\]-\x7F]|\\\[\x00-\x7F]))*(?2)")(?>(?1)\.(?1)(?4))*(?1)@(?!(?1)[a-z\d-]{64,})(?1)(?>([a-z\d](?>[a-z\d-]*[a-z\d])?)(?>(?1)\.(?!(?1)[a-z\d-]{64,})(?1)(?5)){0,126}|\[(?:(?>IPv6:(?>([a-f\d]{1,4})(?>:(?6)){7}|(?!(?:.*[a-f\d][:\]]){8,})((?6)(?>:(?6)){0,6})?::(?7)?))|(?>(?>IPv6:(?>(?6)(?>:(?6)){5}:|(?!(?:.*[a-f\d]:){6,})(?8)?::(?>((?6)(?>:(?6)){0,4}):)?))?(25[0-5]|2[0-4]\d|1\d{2}|[1-9]?\d)(?>\.(?9)){3}))\])(?1)$/isD', $email)===1;*/
		return ((preg_match('/(@.*@)|(\.\.)|(@\.)|(\.@)|(^\.)/', $email)) || (preg_match('/^.+\@(\[?)[a-zA-Z0-9\-\.]+\.([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$/',$email))) && !strpos($email,":");
	}
	
	function youbaze_preorder_html($string)
    {
        return htmlentities($string, ENT_COMPAT, "utf-8");
    }
	
	function youbaze_preorder_isInteger($var)
	{
		if (is_integer($var)) return true;
		if (!is_numeric($var) || is_float($var)) return false;
		if (is_string($var) && strpos($var, '.')!==false) return false;
		return true;
	}
	
	function youbaze_preorder_admin_css()
	{
		wp_enqueue_style('youbaze-preorder-admin-css', plugins_url('css/youbaze_preorder_admin.css', __FILE__), false, '1.0.0', 'all');
	}
	add_action('admin_enqueue_scripts', 'youbaze_preorder_admin_css');
	
	function youbaze_preorder_css()
	{
        wp_enqueue_style('youbaze-preorder-css', plugins_url('css/youbaze_preorder.css', __FILE__), false, '1.0.0', 'all');
    }
    add_action('wp_enqueue_scripts', "youbaze_preorder_css");
	
	class YouBazePreorderWidget extends WP_Widget
	{
		private $jsonString;
		private $attachment;
		
		function __construct() 
		{
			$this->jsonString = '';
			$this->attachment = false;
			parent::__construct('youbaze_preorder', __('YouBazePreorder', 'text_domain'), array('description' => __('YouBaze Preorder Form', 'text_domain')));
		}

		private function getYouBazeSettings()
		{
			return array(
				'contact_type_id' => 'Klant',
				'ordersource_id' => 'Anders',
				'terms_of_payment' => 8,
				'salutation' => 1,
				'mc_language' => 'nl',
				'preorder_project_id' => 1234
			);
		}
		
		private function getFormSettings()
		{
			return array(
				'cssclass' => '',
				'heading' => 'Plaats een pre-order!',
				'submit' => 'Bestel',
				'error_incomplete' => 'Je hebt niet alle verplichte velden ingevoerd.',
				'error_invalidemail' => 'Het e-mailadres is niet juist.',
				'error_invalidvat' => 'Het BTW-nummer is niet juist (er moet bv. NL of BE voor staan).',
				'success' => 'Je pre-order is verstuurd!',
				'select_mediasource_text' => 'Hoe heb je ons gevonden?',
				'company_title_text' => 'Bedrijfsgegevens',
				'personal_title_text' => 'Persoonsgegevens',
				'address_title_text' => 'Adresgegevens',
				'tracking_pixels' => '',
				'conversion_pixels' => ''
			);
		}
		
		private function getEmailSettings()
		{
			return array(
				'sender_name' => 'Naam afzender',
				'sender_email' => 'info@voorbeelddomein.nl',
				'subject' => 'Pre-order',
				'confirmation_email' => "Beste klant,\r\n\r\nBedankt voor het plaatsen van de pre-order. U ontvangt binnen een uur een e-mail waarmee u uw pre-order dient te bevestigen.\r\n\r\nMet vriendelijke groet,\r\n\r\nJouw naam"
			);
		}
		
		private function getMaxProductCount()
		{
			return 4;
		}
		
		private function getProductTextSettings()
		{
			return array(
				'product_quantity_text' => 'Aantal',
				'product_price_text' => 'Prijs',
				'product_moq_text' => 'Minimum aantal',
				'error_noorder' => 'Je hebt geen producten besteld',
				'error_ltmoq' => 'Het ingevoerde aantal is lager dan het minimum aantal'
			);
		}
		
		private function getProductSettings()
		{
			$a = array('shipping_product_id' => '', 'total_moq' => '');
			$maxProducts = $this->getMaxProductCount();
			for ($x=1; $x<=$maxProducts; $x++)
			{
				$a = array_merge($a, array(
					'product'.$x.'_id' => '',
					'product'.$x.'_name' => '',
					'product'.$x.'_price' => '',
					'product'.$x.'_moq' => ''
				));
			}
			return $a;
		}
		
		private function getCompanyFields()
		{
			return array('company', 'department', 'vatnumber', 'cocnumber', 'website');
		}
		
		private function getAddressFields()
		{
			return array('street', 'house_number', 'house_number_suffix', 'address2', 'zipcode', 'city', 'state', 
						 'country', 'pobox', 'pobox_zipcode', 'pobox_city');
		}
		
		private function getOtherFields()
		{
			return array('subject', 'text', 'attachment');
		}
		
		private function getFormFields()
		{
			return array(
				//company
				'company_text' => 'Bedrijfsnaam', 'company_enabled' => 0, 'company_mandatory' => 0,
				'department_text' => 'Afdeling', 'department_enabled' => 0, 'department_mandatory' => 0,
				'vatnumber_text' => 'BTW-nummer', 'vatnumber_enabled' => 1, 'vatnumber_mandatory' => 0,
				'cocnumber_text' => 'KvK-nummer', 'cocnumber_enabled' => 0, 'cocnumber_mandatory' => 0,
				'website_text' => 'Website', 'website_enabled' => 0, 'website_mandatory' => 0,
				
				//personal
				'gender_text' => 'Geslacht', 'gender_enabled' => 1, 'gender_mandatory' => 1,
				'gender_male_text' => 'Man',
				'gender_female_text' => 'Vrouw',
				'firstname_text' => 'Voornaam', 'firstname_enabled' => 0, 'firstname_mandatory' => 0,
				'initials_text' => 'Voorletters', 'initials_enabled' => 1, 'initials_mandatory' => 1,
				'insertion_text' => 'Tussenvoegsel', 'insertion_enabled' => 1, 'insertion_mandatory' => 0,
				'lastname_text' => 'Achternaam', 'lastname_enabled' => 1, 'lastname_mandatory' => 1,
				'function_text' => 'Functie', 'function_enabled' => 0, 'function_mandatory' => 0,
				'email_text' => 'E-mail', 'email_enabled' => 1, 'email_mandatory' => 1,
				'email_general_text' => 'E-mail algemeen', 'email_general_enabled' => 0, 'email_general_mandatory' => 0,
				'phone_text' => 'Telefoonnummer', 'phone_enabled' => 0, 'phone_mandatory' => 0,
				'phone_direct_text' => 'Rechtstreeks nummer', 'phone_direct_enabled' => 0, 'phone_direct_mandatory' => 0,
				'mobile_text' => 'Mobiel nummer', 'mobile_enabled' => 0, 'mobile_mandatory' => 0,
				'skype_text' => 'Skype', 'skype_enabled' => 0, 'skype_mandatory' => 0,
				'twitter_text' => 'Twitter', 'twitter_enabled' => 0, 'twitter_mandatory' => 0,
				'facebook_text' => 'Facebook', 'facebook_enabled' => 0, 'facebook_mandatory' => 0,
				'instagram_text' => 'Instagram', 'instagram_enabled' => 0, 'instagram_mandatory' => 0,
				'googleplus_text' => 'Google+', 'googleplus_enabled' => 0, 'googleplus_mandatory' => 0,
				'tumblr_text' => 'Tumblr', 'tumblr_enabled' => 0, 'tumblr_mandatory' => 0,
				'linkedin_text' => 'Linkedin', 'linkedin_enabled' => 0, 'linkedin_mandatory' => 0, 			
				'blog_text' => 'Blog', 'blog_enabled' => 0, 'blog_mandatory' => 0,
				
				//address
				'street_text' => 'Adres', 'street_enabled' => 0, 'street_mandatory' => 0,
				'house_number_text' => 'Huisnummer', 'house_number_enabled' => 0, 'house_number_mandatory' => 0,
				'house_number_suffix_text' => 'Toevoegsel', 'house_number_suffix_enabled' => 0, 'house_number_suffix_mandatory' => 0,
				'address2_text' => 'Adres 2', 'address2_enabled' => 0, 'address2_mandatory' => 0,
				'zipcode_text' => 'Postcode', 'zipcode_enabled' => 0, 'zipcode_mandatory' => 0,
				'city_text' => 'Plaats', 'city_enabled' => 0, 'city_mandatory' => 0,
				'state_text' => 'Provincie', 'state_enabled' => 0, 'state_mandatory' => 0,
				'country_text' => 'Land', 'country_enabled' => 1, 'country_mandatory' => 1,
				'default_country' => 'NL',
				'pobox_text' => 'Postbus (alleen nummer)', 'pobox_enabled' => 0, 'pobox_mandatory' => 0,
				'pobox_zipcode_text' => 'Postbus postcode', 'pobox_zipcode_enabled' => 0, 'pobox_zipcode_mandatory' => 0,
				'pobox_city_text' => 'Postbus plaats', 'pobox_city_enabled' => 0, 'pobox_city_mandatory' => 0,
				
				'subject_text' => 'Onderwerp', 'subject_enabled' => 0, 'subject_mandatory' => 0,
				'text_text' => 'Text', 'text_enabled' => 0, 'text_mandatory' => 0,
				'text_text_default' => '',
				'attachment_text' => 'Upload bijlage', 'attachment_enabled' => 0, 'attachment_mandatory' => 0
			);
		}
		
		private function getFieldLabel($field)
		{
			$labels = array(
				'contact_type_id' => 'Groeps-ID (zie setting "groepen" in YouBaze)',
				'ordersource_id' => 'Broncode; 1 per regel, als volgt opgebouwd: "id;omschrijving" (zie setting "broncodes" in YouBaze)',
				'terms_of_payment' => 'Betaaltermijn',
				'salutation' => 'Aanhef',
				'mc_language' => 'Taalcode (ISO 639-1)',
				'preorder_project_id' => 'Pre-orderproject ID',
				
				'subject' => 'Onderwerp e-mail',
				'sender_name' => 'Naam afzender',
				'sender_email' => 'E-mail afzender',
				'confirmation_email' => 'Tekst voor bevestigingsmail',
				
				'cssclass' => 'Extra CSS class(es)',
				'heading' => 'Kopregel boven pre-orderformulier',
				'submit' => 'Tekst op verzendknop',
				'error_incomplete' => 'Melding niet alle verplichte velden ingevoerd',
				'error_invalidemail' => 'Melding fout e-mailadres',
				'error_invalidvat' => 'Melding fout BTW-nummer',
				'success' => 'Melding pre-order succesvol verwerkt',
				'select_mediasource_text' => 'Select media source title',
				'company_title_text' => 'Subkop bedrijfsgegevens',
				'personal_title_text' => 'Subkop persoonsgegevens',
				'address_title_text' => 'Subkop adresgegevens',
				'tracking_pixels' => 'Trackingpixels',
				'conversion_pixels' => 'Conversiepixels',
				
				'product_quantity_text' => 'Aantal',
				'product_price_text' => 'Prijs',
				'product_moq_text' => 'Minimum orderaantal',
				'error_noorder' => 'Melding geen producten besteld',
				'error_ltmoq' => 'Melding besteld aantal lager dan minimum aantal',
				
				'gender_text' => 'Geslacht',
				'gender_male_text' => 'Geslacht man',
				'gender_female_text' => 'Geslacht vrouw',
				'firstname_text' => 'Voornaam',
				'initials_text' => 'Voorletters',
				'insertion_text' => 'Tussenvoegsel',
				'lastname_text' => 'Achternaam',
				'function_text' => 'Functie',
				'zipcode_text' => 'Postcode',
				'house_number_text' => 'Huisnummer',
				'house_number_suffix_text' => 'Toevoegsel',
				'street_text' => 'Adres',
				'address2_text' => 'Adres 2',
				'city_text' => 'Plaats',
				'state_text' => 'Provincie',
				'country_text' => 'Land',
				'default_country' => 'Vooringevuld land',
				'pobox_text' => 'Postbus',
				'pobox_zipcode_text' => 'Postbus postcode',
				'pobox_city_text' => 'Postbus plaats',
				'company_text' => 'Bedrijfsnaam', 
				'department_text' => 'Afdeling',
				'cocnumber_text' => 'KvK-nummer', 
				'vatnumber_text' => 'BTW-nummer',
				'email_text' => 'E-mail',
				'email_general_text' => 'E-mail algemeen',
				'phone_text' => 'Telefoonnummer',
				'phone_direct_text' => 'Rechstreeks nummer',
				'mobile_text' => 'Mobiel nummer', 
				'skype_text' => 'Skype',
				'website_text' => 'Website',
				'twitter_text' => 'Twitter', 
				'facebook_text' => 'Facebook',
				'instagram_text' => 'Instagram',
				'googleplus_text' => 'Google+',
				'tumblr_text' => 'Tumblr',
				'linkedin_text' => 'Linkedin',
				'blog_text' => 'Blog',
				'subject_text' => 'Onderwerp open tekstveld',
				'text_text' => 'Text (eg. Question/Remark/Suggestion)',
				'text_text_default' => 'Vooringevulde tekst open tekstveld',
				'attachment_text' => 'Tekst voor bijlage (er kan slechts 1 bestand worden geupload)',
				'shipping_product_id' => 'Product ID voor verzendkosten (leeg laten indien gratis verzending)',
				'total_moq' => 'Totaal minimum aantal'
			);
			
			$maxProducts = $this->getMaxProductCount();
			for ($x=1; $x<=$maxProducts; $x++)
			{
				$labels = array_merge($labels, array(
					'product'.$x.'_id' => 'Product '.$x.' YouBaze ID',
					'product'.$x.'_name' => 'Product '.$x.' naam',
					'product'.$x.'_price' => 'Product '.$x.' prijs',
					'product'.$x.'_moq' => 'Product '.$x.' minimum aantal'
				));
			}
			
			return isset($labels[$field]) ? $labels[$field] : '';
		}
		
		public function form($instance) 
		{
			$youbazeSettings = $this->getYouBazeSettings();
			$formSettings = $this->getFormSettings();
			$emailSettings = $this->getEmailSettings();
			$productSettings = $this->getProductSettings();
			$productTextSettings = $this->getProductTextSettings();
			$formFields = $this->getFormFields();
			$array = array_merge($youbazeSettings, $formSettings, $emailSettings, $productSettings, $productTextSettings, $formFields);
			$countries = $this->getCountries();
			$instance = wp_parse_args((array) $instance, $array);
			$optionvalue = get_option(youbaze_preorder_getOptionName());
			
			?><h3>YouBaze Pre-ordersettings</h3><?php
			$hasProduct = false;
			foreach ($productSettings as $k => $v)
			{
				if (substr($k, -3)!='_id') continue;
				if ($instance[$k]!='' && youbaze_preorder_isInteger($instance[$k]))
				{
					$hasProduct = true;
					break;
				}
			}
			if (!$hasProduct)
			{
				?>
				<div class="youbaze_notification"><p><strong><?php echo youbaze_preorder_html('De widget zal niet zichtbaar zijn zolang je geen productgegevens hebt ingevoerd.');?></strong></p></div>
				<?php
			}
			foreach ($youbazeSettings as $key => $value)
			{
				if ($key!='preorder_project_id') continue;
				?>
				<p>
					<label for="<?php echo $this->get_field_id($key); ?>"><?php echo youbaze_preorder_html($this->getFieldLabel($key));?></label>
					<input type="text" class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" value="<?php echo youbaze_preorder_html($instance[$key]); ?>" style="width:100%;" />
				</p>
				<?php
			}
			?><h3>Productsettings</h3><?php
			foreach ($productTextSettings as $key => $value)
			{
				?>
				<p>
					<label for="<?php echo $this->get_field_id($key); ?>"><?php echo youbaze_preorder_html($this->getFieldLabel($key));?></label>
					<input type="text" class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" value="<?php echo youbaze_preorder_html($instance[$key]); ?>" style="width:100%;" />
				</p>
				<?php
			}
			?><h3>Product settings</h3><?php
			foreach ($productSettings as $key => $value)
			{
				$n = $this->get_field_id($key);
				if (substr($n, -4)=='_moq' || substr($n, -3)=='_id')
				{
					?>
					<p>
						<label for="<?php echo $this->get_field_id($key); ?>"><?php echo youbaze_preorder_html($this->getFieldLabel($key));?></label>
						<input type="number" class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" value="<?php echo youbaze_preorder_html($instance[$key]); ?>" style="width:100%;" min="1" />
					</p>
					<?php
				} else
				{
					?>
					<p>
						<label for="<?php echo $this->get_field_id($key); ?>"><?php echo youbaze_preorder_html($this->getFieldLabel($key));?></label>
						<input type="text" class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" value="<?php echo youbaze_preorder_html($instance[$key]); ?>" style="width:100%;" />
					</p>
					<?php
				}
			}
			?><h3>Settings contactgegevens</h3><?php
			foreach ($youbazeSettings as $key => $value)
			{
				if ($key=='preorder_project_id') continue;
				?>
				<p>
					<label for="<?php echo $this->get_field_id($key); ?>"><?php echo youbaze_preorder_html($this->getFieldLabel($key));?></label>
					<?php
						if ($key=='salutation')
						{
							?>
							<select class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>">
								<option value="1"<?php echo isset($instance[$key]) && $instance[$key]==1 ? ' selected="selected"' : '';?>>Voornaam (Beste John,)</option>
								<option value="2"<?php echo isset($instance[$key]) && $instance[$key]==2 ? ' selected="selected"' : '';?>>Achternaam (Geachte heer Jansen,)</option>
							</select>
							<?php
						} elseif ($key=='ordersource_id')
						{
							?>
							<textarea class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" cols="8" rows="16"><?php echo youbaze_preorder_html($instance[$key]); ?></textarea>
							<?php
							
						} else
						{
							?>
							<input type="text" class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" value="<?php echo youbaze_preorder_html($instance[$key]); ?>" style="width:100%;" />
							<?php
						}
					?>
				</p>
				<?php
			}
			?><h3>E-mail settings</h3><?php
			foreach ($emailSettings as $key => $value)
			{
				?>
				<p>
					<label for="<?php echo $this->get_field_id($key); ?>"><?php echo youbaze_preorder_html($this->getFieldLabel($key));?></label>
					<?php
						if ($key=='confirmation_email')
						{
							?>
							<textarea class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" cols="20" rows="16"><?php echo youbaze_preorder_html($instance[$key]); ?></textarea>
							<?php
						} else
						{
							?>
							<input type="text" class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" value="<?php echo youbaze_preorder_html($instance[$key]); ?>" style="width:100%;" />
							<?php
						}
					?>
				</p>
				<?php
			}
			?><h3>Meldingen, regels en kopregels</h3><?php
			foreach ($formSettings as $key => $value)
			{
				?>
				<p>
					<label for="<?php echo $this->get_field_id($key); ?>"><?php echo youbaze_preorder_html($this->getFieldLabel($key));?></label>
					<?php
						if (in_array($key, array('conversion_pixels', 'tracking_pixels')))
						{
							?>
							<textarea class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" cols="8" rows="16"><?php echo youbaze_preorder_html($instance[$key]); ?></textarea>
							<?php
						} else
						{
							?>
							<input type="text" class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" value="<?php echo youbaze_preorder_html($instance[$key]); ?>" style="width:100%;" />
							<?php
						}
						if ($key=='cssclass')
						{
							?>
							<br /><small>Als je extra CSS class(es) toevoegt, dan kun je het design van het pre-orderformulier aanpassen via CSS.</small>
							<?php
						}
					?>
				</p>
				<?php
			}
			?>
			<h3>Velden in pre-orderformulier</h3>
			<h4>BEDRIJFSGEGEVENS</h4>
			<?php
			foreach ($formFields as $key => $value)
			{
				if (strpos($key, '_enabled')!==false || strpos($key, '_mandatory')!==false) continue;
				$orgkey = $key;
				?>
				<p>
					<label for="<?php echo $this->get_field_id($key); ?>"><?php echo youbaze_preorder_html($this->getFieldLabel($key));?></label>
					<?php
						if ($key=='text_text_default')
						{
							?>
							<textarea class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" cols="20" rows="16"><?php echo youbaze_preorder_html($instance[$key]); ?></textarea>
							<?php
						} elseif ($key=='default_country')
						{
							?>
							<select class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" style="width: 100%;">
								<option value=""></option>
								<?php
									foreach ($countries as $cc => $cname)
									{
										?>
										<option value="<?php echo youbaze_preorder_html($cc)?>"<?php echo isset($instance[$key]) && $instance[$key]==$cc ? ' selected="selected"' : '';?>><?php echo youbaze_preorder_html($cname)?></option>
										<?php
									}
								?>
							</select>
							<?php
						} else
						{
							$style = $key=='text_text' ? ' style="display: none;"' : '';
							?>
							<input<?php echo $style;?> type="text" class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" value="<?php echo youbaze_preorder_html($instance[$key]); ?>" style="width:100%;" />
							<?php
							if ($key=='text_text') echo '<br />';
						}
					
						if (array_key_exists($key, $formFields) && !in_array($key, array('gender_female_text', 'gender_male_text', 'text_text_default', 'default_country')))
						{
							$key = str_replace('_text', '_enabled', $key);
							?>
							<input<?php echo isset($instance[$key]) && $instance[$key]==1 ? ' checked="checked"' : '';?> id="<?php echo $this->get_field_id($key); ?>" type="checkbox" value="1" name="<?php echo $this->get_field_name($key); ?>" />
							<label for="<?php echo $this->get_field_id($key); ?>">Zichtbaar?</label>&nbsp;&nbsp;
							<?php
							$key = str_replace('_enabled', '_mandatory', $key);
							?>
							<input<?php echo isset($instance[$key]) && $instance[$key]==1 ? ' checked="checked"' : '';?> id="<?php echo $this->get_field_id($key); ?>" type="checkbox" value="1" name="<?php echo $this->get_field_name($key); ?>" />
							<label for="<?php echo $this->get_field_id($key); ?>">Verplicht?</label>
							<?php
						}
					?>
				</p>
				<?php
				if ($orgkey=='website_text')
				{
					?><h4>PERSOONSGEGEVENS</h4><?php
					
				} elseif ($orgkey=='blog_text')
				{
					?><h4>ADRESGEGEVENS</h4><?php
					
				} elseif ($orgkey=='text_text_default')
				{
					?><h4>BIJLAGE OM TE UPLOADEN</h4><?php
				}
			}
		}
		
		public function update($new_instance, $old_instance) 
		{
			$youbazeSettings = $this->getYouBazeSettings();
			$formSettings = $this->getFormSettings();
			$emailSettings = $this->getEmailSettings();
			$productSettings = $this->getProductSettings();
			$productTextSettings = $this->getProductTextSettings();
			$formFields = $this->getFormFields();
			$array = array_merge($youbazeSettings, $formSettings, $emailSettings, $productSettings, $productTextSettings, $formFields);
			
			$instance = $old_instance;
			foreach ($array as $key => $value)
				$instance[$key] = $new_instance[$key];
			
			$maxProducts = $this->getMaxProductCount();
			for ($x=1; $x<=$maxProducts; $x++)
			{
				if (isset($instance['product'.$x.'_id']) && !(youbaze_preorder_isInteger($instance['product'.$x.'_id']) || $instance['product'.$x.'_id']==''))
					$instance['product'.$x.'_id'] = $old_instance['product'.$x.'_id'];
				
				if (isset($instance['product'.$x.'_moq']) && !((youbaze_preorder_isInteger($instance['product'.$x.'_moq']) && $instance['product'.$x.'_moq']>=0) || $instance['product'.$x.'_moq']==''))
					$instance['product'.$x.'_moq'] = $old_instance['product'.$x.'_moq']=='' && $instance['product'.$x.'_id']!='' ? 0 : $old_instance['product'.$x.'_moq'];
				
				if (isset($instance['product'.$x.'_id']) && $instance['product'.$x.'_id']!='')
				{
					if (isset($instance['product'.$x.'_name']) && trim($instance['product'.$x.'_name']==''))
						$instance['product'.$x.'_name'] = 'Product';
				
					if (isset($instance['product'.$x.'_moq']) && !youbaze_preorder_isInteger($instance['product'.$x.'_moq']))
						$instance['product'.$x.'_moq'] = 0;
				}
			}
					
			return $instance;
		}
		
		public function youbaze_phpmailer($phpmailer) 
		{
			$phpmailer->isHTML(true);
			$phpmailer->addStringAttachment($this->jsonString, 'preorder.youbaze', 'base64', 'application/json', 'attachment');
			if ($this->attachment!==false)
				$phpmailer->AddAttachment($this->attachment['tmp_name'], $this->attachment['name']);
		}
		
		public function widget($args, $instance)
		{
			$optionvalue = get_option(youbaze_preorder_getOptionName());
			$checkFields = array();
			$products = array();
			$formFields = $this->getFormFields();
			$productSettings = $this->getProductSettings();
			$countries = $this->getCountries();
			$ordersource_id = isset($_POST[$this->id.'-youbaze_submit'], $_POST['youbaze_ordersource_id']) ? trim($_POST['youbaze_ordersource_id']) : '';
			foreach ($formFields as $key => $value)
			{
				if (strpos($key, '_enabled')!==false || strpos($key, '_mandatory')!==false) continue;
				if (in_array($key, array('gender_female_text', 'gender_male_text', 'text_text_default', 'attachment_text'))) continue;
				$enabledKey = str_replace('_text', '_enabled', $key);
				if ($instance[$enabledKey]==0) continue;
				$$key = isset($_POST[$this->id.'-youbaze_submit'], $_POST['youbaze_'.$key]) ? trim($_POST['youbaze_'.$key]) : '';
				$checkFields[] = $key;
			}
			
			$sent = false;
			$error = false;
			$errorfields = array();
			
			$maxProducts = $this->getMaxProductCount();
			for ($x=1; $x<=$maxProducts; $x++)
			{
				if (!($instance['product'.$x.'_id']!='' && youbaze_preorder_isInteger($instance['product'.$x.'_id']))) continue;
				$key = 'product'.$x.'_quantity';
				$$key = isset($_POST[$this->id.'-youbaze_submit'], $_POST['youbaze_'.$key]) ? trim($_POST['youbaze_'.$key]) : '';
			}
			
			if (isset($_POST[$this->id.'-youbaze_submit']))
			{
				for ($x=1; $x<=$maxProducts; $x++)
				{
					if (!($instance['product'.$x.'_id']!='' && youbaze_preorder_isInteger($instance['product'.$x.'_id']))) continue;
					$key = 'product'.$x.'_quantity';
					if ($$key=='') continue;
					if (!(youbaze_preorder_isInteger($$key) && $$key>=0))
					{
						$error = $instance['error_incomplete'];
						$errorfields[] = $key;

					} elseif (youbaze_preorder_isInteger($instance['product'.$x.'_moq']) && $$key<$instance['product'.$x.'_moq'] && $$key!=0)
					{
						$error = $instance['error_ltmoq'];
						$errorfields[] = $key;

					} else
					{
						$products[$instance['product'.$x.'_id']] = $$key;
					}
				}
				
				foreach ($checkFields as $checkField)
				{
					$mandatory = $instance[str_replace('_text', '_mandatory', $checkField)]==1;
					if ($mandatory && $$checkField=='')
					{
						$error = $instance['error_incomplete'];
						$errorfields[] = $checkField;
					}
				}
				if ($ordersource_id=='')
				{
					$error = $instance['error_incomplete'];
					$errorfields[] = 'ordersource_id';
				}
				if ($error===false && in_array('email_text', $checkFields) && $email_text!='' && !youbaze_preorder_validateEmail($email_text))
				{
					$error = $instance['error_invalidemail'];
					$errorfields[] = 'email_text';
				}
				if ($error===false && in_array('email_general_text', $checkFields) && $email_general_text!='' && !youbaze_preorder_validateEmail($email_general_text))
				{
					$error = $instance['error_invalidemail'];
					$errorfields[] = 'email_general_text';
				}
				
				if (in_array('vatnumber_text', $checkFields) && $vatnumber_text!='')
					$vatnumber_text = YouBazePreorderVIES::formatVATNumber($vatnumber_text);
				
				if ($error===false && 
					in_array('vatnumber_text', $checkFields) && 
					in_array('country_text', $checkFields) && 
					YouBazePreorderVIES::isEUMemberState($country_text) && 
					$vatnumber_text!='')
				{
					$validVAT = false;
					$strippedVATnumber = YouBazePreorderVIES::stripVATNumber($vatnumber_text);
					if ($strippedVATnumber!=$vatnumber_text)
					{
						$vies = new YouBazePreorderVIES();
						if ($vies->validate($country_text, $vatnumber_text))
							$validVAT = true;
					}
					if (!$validVAT)
					{
						$error = $instance['error_invalidvat'];
						$errorfields[] = 'vatnumber_text';
					}
				}
				
				if ($error===false/* && count($products)>0*/)
				{
					$totalProductCount = 0;
					foreach ($products as $id => $c)
						$totalProductCount += $c;
					if ($c===0)
						$error = $instance['error_noorder'];
					elseif (youbaze_preorder_isInteger($instance['total_moq']) && $c<$instance['total_moq'])
						$error = $instance['error_ltmoq'].' ('.$instance['total_moq'].')';
				}
				
				if ($error===false)
				{
					if ($instance['attachment_enabled']==1)
					{
						if (isset($_FILES['youbaze_attachment_text']) && is_uploaded_file($_FILES['youbaze_attachment_text']['tmp_name']))
						{
							if (!in_array($_FILES['youbaze_attachment_text']['name'], array('.htaccess', 'contact.youbaze', 'preorder.youbaze')))
								$this->attachment = $_FILES['youbaze_attachment_text'];
						}
					}
					if ($this->attachment===false && $instance['attachment_mandatory']==1)
					{
						$error = $instance['error_incomplete'];
						$errorfields[] = 'attachment_text';
					}
					
					if ($error===false)
					{
						if (in_array('email_text', $checkFields) && $email_text!='')
						wp_mail($email_text, $instance['subject'], nl2br(youbaze_preorder_html($instance['confirmation_email'])), array('Content-Type: text/html; charset=UTF-8', 'From: "'.$instance['sender_name'].'" <'.$instance['sender_email'].'>'));

						$json = array();
						$mailData = array();
						foreach ($this->getYouBazeSettings() as $k => $v)
							$json[$k] = $instance[$k];
						
						$json['ordersource_id'] = $ordersource_id;
						
						for ($x=1; $x<=$maxProducts; $x++)
						{
							if (!array_key_exists($instance['product'.$x.'_id'], $products)) continue;
							if ($products[$instance['product'.$x.'_id']]==0) continue;
							
							$label = $products[$instance['product'.$x.'_id']].'x '.$instance['product'.$x.'_name'];
							$extraLabelParts = array();
							if ($instance['product'.$x.'_price']!='')
								$extraLabelParts[] = ($instance['product_price_text']!='' ? $instance['product_price_text'].': ' : '').$instance['product'.$x.'_price'];
							if ($instance['product'.$x.'_moq']!='')
								$extraLabelParts[] = ($instance['product_moq_text']!='' ? $instance['product_moq_text'].': ' : '').$instance['product'.$x.'_moq'];
							if (count($extraLabelParts)>0)
								$label .= ' ('. implode(', ', $extraLabelParts).')';
							$mailData[] = $label;
						}
						$body = in_array('text_text', $checkFields) && $text_text!='' ? nl2br($text_text).'<br /><br /><hr /><br /><br />' : '';
						$body .= implode('<br />', $mailData);
						$body .= '<br /><br /><hr /><br /><br />';
						$mailData = array();
						foreach ($checkFields as $checkField)
						{
							if (in_array($checkField, array('text_text', 'subject_text'))) continue;
							$json[str_replace('_text', '', $checkField)] = $$checkField;
							$v = $checkField=='gender_text' ? ($$checkField==1 ? $instance['gender_male_text'] : $instance['gender_female_text']) : $$checkField;
							$mailData[] = $instance[$checkField].': '.$v;
						}
						$body .= implode('<br />', $mailData);
						if (youbaze_preorder_isInteger($instance['shipping_product_id']) && $instance['shipping_product_id']>0)
						{
							if (!isset($products[$instance['shipping_product_id']]))
								$products[$instance['shipping_product_id']] = 1;
						}							
						$json['products'] = $products;
						$this->jsonString = json_encode($json);
						
						add_action('phpmailer_init', array($this, 'youbaze_phpmailer'));
						$subject = in_array('subject_text', $checkFields) && $subject_text!='' ? $subject_text : $instance['subject'];
						wp_mail($optionvalue['youbaze_email'], $subject, $body, 'From: "'.$instance['sender_name'].'" <'.$instance['sender_email'].'>');
						$this->jsonString = '';
						$this->attachment = false;

						foreach ($checkFields as $checkField)
							$$checkField = '';
						$text_text = $instance['text_text_default'];
						for ($x=1; $x<=$maxProducts; $x++)
						{
							if (!($instance['product'.$x.'_id']!='' && youbaze_preorder_isInteger($instance['product'.$x.'_id']))) continue;
							$key = 'product'.$x.'_quantity';
							$$key = '';//$$key = 0;
						}
						$ordersource_id = '';
						
						$sent = true;
					}
				}
				
			} else
			{
				$gender = '';
				$text_text = $instance['text_text_default'];
				$country_text = $instance['default_country'];
			}
			
			extract($args, EXTR_SKIP);
			
			$hasProduct = false;
			foreach ($productSettings as $k => $v)
			{
				if (substr($k, -3)!='_id') continue;
				if ($instance[$k]!='' && youbaze_preorder_isInteger($instance[$k]))
				{
					$hasProduct = true;
					break;
				}
			}
			if (!$hasProduct) return;
			
			echo $before_widget;
			?>			
			<div id="my<?php echo $this->id;?>" class="youbazewidget<?php echo $instance['cssclass']=='' ? '' : ' '.youbaze_preorder_html($instance['cssclass'])?>">
				<h2 class="widget-title"><?=youbaze_preorder_html($instance['heading'])?></h2>
				<?php
					if ($sent)
					{
						?><div class="youbaze_success"><?php echo youbaze_preorder_html($instance['success'])?></div><?php
						
					} elseif ($error!==false)
					{
						?><div class="youbaze_error"><?php echo youbaze_preorder_html($error)?></div><?php
					}
				?>				
				<form method="post" enctype="multipart/form-data" action="#my<?php echo $this->id;?>">
					<fieldset>
						<?php
							for ($x=1; $x<=$maxProducts; $x++)
							{
								if (!($instance['product'.$x.'_id']!='' && youbaze_preorder_isInteger($instance['product'.$x.'_id']))) continue;
								$key = 'product'.$x.'_quantity';
								$errorclass = in_array($key, $errorfields) ? ' class="youbaze_errorfield text_input"' : ' class="text_input"';
								$label = /*$instance['product_quantity_text'].' - '.*/$instance['product'.$x.'_name'];
								$extraLabelParts = array();
								if ($instance['product'.$x.'_price']!='')
									$extraLabelParts[] = ($instance['product_price_text']!='' ? $instance['product_price_text'].': ' : '').$instance['product'.$x.'_price'];
								if ($instance['product'.$x.'_moq']!='')
									$extraLabelParts[] = ($instance['product_moq_text']!='' ? $instance['product_moq_text'].': ' : '').$instance['product'.$x.'_moq'];
								/*if (count($extraLabelParts)>0)
									$label .= ' ('. implode(', ', $extraLabelParts).')';*/
								?>
								<div class="youbaze_section product<?php echo $x?>">
									<p>
										<label for="youbaze_<?php echo youbaze_preorder_html($key);?>"><?php echo youbaze_preorder_html($label);?></label><br />
										<input<?php echo $errorclass;?> type="text" name="youbaze_<?php echo youbaze_preorder_html($key);?>" id="youbaze_<?php echo youbaze_preorder_html($key);?>" value="<?php echo youbaze_preorder_html($$key);?>" min="0" placeholder="<?php echo youbaze_preorder_html($instance['product_quantity_text']);?><?php echo youbaze_preorder_isInteger($instance['product'.$x.'_moq']) && $instance['product'.$x.'_moq']>0 ? '' : ''?>" />
									</p>
								</div>
								<?php
							}
							
							?>
							<div class="youbaze_section">
								<p>
									<?php
										$key = 'ordersource_id';
										$errorclass = in_array($key, $errorfields) ? ' class="youbaze_errorfield text_input"' : ' class="text_input"';
										$orderSources = explode("\r\n", $instance['ordersource_id']);
									?>
									<select<?php echo $errorclass;?> name="youbaze_<?php echo youbaze_preorder_html($key);?>" id="youbaze_<?php echo youbaze_preorder_html($key);?>">
										<option value=""><?php echo youbaze_preorder_html($instance['select_mediasource_text']);?>*</option>
										<?php
											foreach ($orderSources as $orderSource)
											{
												$parts = explode(';', $orderSource);
												if (count($parts)<2) continue;
												$val = trim($parts[0]);
												unset($parts[0]);
												$lab = trim(implode(';', $parts));
												?>
												<option value="<?php echo youbaze_preorder_html($val);?>"<?php echo $ordersource_id==$val ? ' selected="selected"' : '';?>><?php echo youbaze_preorder_html($lab);?></option>
												<?php
											}
										?>
									</select>
								</p>
							</div>
							<?php
							
							$addressTitleShown = false;
							$personalTitleShown = false;
							$companyTitleShown = false;
							
							foreach ($formFields as $key => $value)
							{
								if (strpos($key, '_enabled')!==false || strpos($key, '_mandatory')!==false) continue;
								if (in_array($key, array('gender_female_text', 'gender_male_text', 'text_text_default'))) continue;
								$enabledKey = str_replace('_text', '_enabled', $key);
								if ($instance[$enabledKey]==0) continue;
								$mandatory = $instance[str_replace('_text', '_mandatory', $key)]==1;
								$errorclass = in_array($key, $errorfields) ? ' class="youbaze_errorfield text_input"' : ' class="text_input"';
								$k = str_replace('_text', '', $key);
								if (!$companyTitleShown && in_array($k, $this->getCompanyFields()) && $instance['company_title_text']!='')
								{
									?><h4><?php echo $instance['company_title_text']?></h4><?
									$companyTitleShown = true;
									
								} elseif (!$personalTitleShown && !in_array($k, array_merge($this->getCompanyFields(), $this->getAddressFields(), $this->getOtherFields())) && $instance['personal_title_text']!='')
								{
									?><h4><?php echo $instance['personal_title_text']?></h4><?
									$personalTitleShown = true;
									
								} elseif (!$addressTitleShown && in_array($k, $this->getAddressFields()) && $instance['address_title_text']!='')
								{
									?><h4><?php echo $instance['address_title_text']?></h4><?
									$addressTitleShown = true;
								}
								?>
								<div class="youbaze_section <?php echo $key?>">
									<p>
										<?/*<label for="youbaze_<?php echo youbaze_preorder_html($key);?>"><?php echo youbaze_preorder_html($instance[$key]);?><?php echo $mandatory ? '<i class="required">*</i>' : ''?></label>*/?>
										<?php
											if ($key=='text_text')
											{
												?>
												<textarea<?php echo $errorclass;?> name="youbaze_<?php echo youbaze_preorder_html($key);?>" id="youbaze_<?php echo youbaze_preorder_html($key);?>" rows="12" placeholder="<?php echo youbaze_preorder_html($instance[$key]);?><?php echo $mandatory ? '*' : ''?>"><?php echo youbaze_preorder_html($$key);?></textarea>
												<?php

											} elseif ($key=='gender_text')
											{
												?>
												<select<?php echo $errorclass;?> name="youbaze_<?php echo youbaze_preorder_html($key);?>" id="youbaze_<?php echo youbaze_preorder_html($key);?>">
													<option value=""><?php echo youbaze_preorder_html($instance[$key]);?><?php echo $mandatory ? '*' : ''?></option>
													<option value="1"<?php echo $$key==1 ? ' selected="selected"' : '';?>><?php echo youbaze_preorder_html($instance['gender_male_text']);?></option>
													<option value="2"<?php echo $$key==2 ? ' selected="selected"' : '';?>><?php echo youbaze_preorder_html($instance['gender_female_text']);?></option>
												</select>
												<?php
												
											} elseif ($key=='country_text')
											{
												?>
												<select<?php echo $errorclass;?> name="youbaze_<?php echo youbaze_preorder_html($key);?>" id="youbaze_<?php echo youbaze_preorder_html($key);?>">
													<option value=""><?php echo youbaze_preorder_html($instance[$key]);?><?php echo $mandatory ? '*' : ''?></option>
													<?php
														foreach ($countries as $cc => $cname)
														{
															?>
															<option value="<?php echo youbaze_preorder_html($cc)?>"<?php echo $$key==$cc ? ' selected="selected"' : '';?>><?php echo youbaze_preorder_html($cname)?></option>
															<?php
														}
													?>
												</select>
												<?php	
												
											} elseif ($key=='attachment_text')
											{
												?>
												<label for="youbaze_<?php echo youbaze_preorder_html($key);?>"><?php echo youbaze_preorder_html($instance[$key]);?><?php echo $mandatory ? '<i class="required">*</i>' : ''?></label><br />
												<input<?php echo $errorclass;?> type="file" name="youbaze_<?php echo youbaze_preorder_html($key);?>" id="youbaze_<?php echo youbaze_preorder_html($key);?>" />
												<?php
											} else
											{
												?>
												<input<?php echo $errorclass;?> type="text" name="youbaze_<?php echo youbaze_preorder_html($key);?>" id="youbaze_<?php echo youbaze_preorder_html($key);?>" value="<?php echo youbaze_preorder_html($$key);?>" placeholder="<?php echo youbaze_preorder_html($instance[$key]);?><?php echo $mandatory ? '*' : ''?>" />
												<?php
											}
										?>
									</p>
								</div>
								<?php
							}
						?>
						<div class="youbaze_buttons">
							<input class="button" type="submit" name="<?php echo $this->id;?>-youbaze_submit" value="<?php echo youbaze_preorder_html($instance['submit']);?>" />
						</div>
					</fieldset>
				</form>
			</div>
			<?php
			echo $sent ? $instance['conversion_pixels'] : $instance['tracking_pixels'];
			echo $after_widget;
		}
		
		private function getCountries($cc = '')
		{
			$countries = array
			(
				'AF' => 'Afghanistan',
				'AX' => 'Aland Islands',
				'AL' => 'Albania',
				'DZ' => 'Algeria',
				'AS' => 'American Samoa',
				'AD' => 'Andorra',
				'AO' => 'Angola',
				'AI' => 'Anguilla',
				'AQ' => 'Antarctica',
				'AG' => 'Antigua And Barbuda',
				'AR' => 'Argentina',
				'AM' => 'Armenia',
				'AW' => 'Aruba',
				'AU' => 'Australia',
				'AT' => 'Austria',
				'AZ' => 'Azerbaijan',
				'BS' => 'Bahamas',
				'BH' => 'Bahrain',
				'BD' => 'Bangladesh',
				'BB' => 'Barbados',
				'BY' => 'Belarus',
				'BE' => 'Belgium',
				'BZ' => 'Belize',
				'BJ' => 'Benin',
				'BM' => 'Bermuda',
				'BT' => 'Bhutan',
				'BO' => 'Bolivia',
				'BA' => 'Bosnia And Herzegovina',
				'BW' => 'Botswana',
				'BV' => 'Bouvet Island',
				'BR' => 'Brazil',
				'IO' => 'British Indian Ocean Territory',
				'BN' => 'Brunei Darussalam',
				'BG' => 'Bulgaria',
				'BF' => 'Burkina Faso',
				'BI' => 'Burundi',
				'KH' => 'Cambodia',
				'CM' => 'Cameroon',
				'CA' => 'Canada',
				'CV' => 'Cape Verde',
				'KY' => 'Cayman Islands',
				'CF' => 'Central African Republic',
				'TD' => 'Chad',
				'CL' => 'Chile',
				'CN' => 'China',
				'CX' => 'Christmas Island',
				'CC' => 'Cocos (Keeling) Islands',
				'CO' => 'Colombia',
				'KM' => 'Comoros',
				'CG' => 'Congo',
				'CD' => 'Congo, Democratic Republic',
				'CK' => 'Cook Islands',
				'CR' => 'Costa Rica',
				'CI' => 'Cote D\'Ivoire',
				'HR' => 'Croatia',
				'CU' => 'Cuba',
				'CY' => 'Cyprus',
				'CZ' => 'Czech Republic',
				'DK' => 'Denmark',
				'DJ' => 'Djibouti',
				'DM' => 'Dominica',
				'DO' => 'Dominican Republic',
				'EC' => 'Ecuador',
				'EG' => 'Egypt',
				'SV' => 'El Salvador',
				'GQ' => 'Equatorial Guinea',
				'ER' => 'Eritrea',
				'EE' => 'Estonia',
				'ET' => 'Ethiopia',
				'FK' => 'Falkland Islands (Malvinas)',
				'FO' => 'Faroe Islands',
				'FJ' => 'Fiji',
				'FI' => 'Finland',
				'FR' => 'France',
				'GF' => 'French Guiana',
				'PF' => 'French Polynesia',
				'TF' => 'French Southern Territories',
				'GA' => 'Gabon',
				'GM' => 'Gambia',
				'GE' => 'Georgia',
				'DE' => 'Germany',
				'GH' => 'Ghana',
				'GI' => 'Gibraltar',
				'GR' => 'Greece',
				'GL' => 'Greenland',
				'GD' => 'Grenada',
				'GP' => 'Guadeloupe',
				'GU' => 'Guam',
				'GT' => 'Guatemala',
				'GG' => 'Guernsey',
				'GN' => 'Guinea',
				'GW' => 'Guinea-Bissau',
				'GY' => 'Guyana',
				'HT' => 'Haiti',
				'HM' => 'Heard Island & Mcdonald Islands',
				'VA' => 'Holy See (Vatican City State)',
				'HN' => 'Honduras',
				'HK' => 'Hong Kong',
				'HU' => 'Hungary',
				'IS' => 'Iceland',
				'IN' => 'India',
				'ID' => 'Indonesia',
				'IR' => 'Iran, Islamic Republic Of',
				'IQ' => 'Iraq',
				'IE' => 'Ireland',
				'IM' => 'Isle Of Man',
				'IL' => 'Israel',
				'IT' => 'Italy',
				'JM' => 'Jamaica',
				'JP' => 'Japan',
				'JE' => 'Jersey',
				'JO' => 'Jordan',
				'KZ' => 'Kazakhstan',
				'KE' => 'Kenya',
				'KI' => 'Kiribati',
				'KR' => 'Korea',
				'KW' => 'Kuwait',
				'KG' => 'Kyrgyzstan',
				'LA' => 'Lao People\'s Democratic Republic',
				'LV' => 'Latvia',
				'LB' => 'Lebanon',
				'LS' => 'Lesotho',
				'LR' => 'Liberia',
				'LY' => 'Libyan Arab Jamahiriya',
				'LI' => 'Liechtenstein',
				'LT' => 'Lithuania',
				'LU' => 'Luxembourg',
				'MO' => 'Macao',
				'MK' => 'Macedonia',
				'MG' => 'Madagascar',
				'MW' => 'Malawi',
				'MY' => 'Malaysia',
				'MV' => 'Maldives',
				'ML' => 'Mali',
				'MT' => 'Malta',
				'MH' => 'Marshall Islands',
				'MQ' => 'Martinique',
				'MR' => 'Mauritania',
				'MU' => 'Mauritius',
				'YT' => 'Mayotte',
				'MX' => 'Mexico',
				'FM' => 'Micronesia, Federated States Of',
				'MD' => 'Moldova',
				'MC' => 'Monaco',
				'MN' => 'Mongolia',
				'ME' => 'Montenegro',
				'MS' => 'Montserrat',
				'MA' => 'Morocco',
				'MZ' => 'Mozambique',
				'MM' => 'Myanmar',
				'NA' => 'Namibia',
				'NR' => 'Nauru',
				'NP' => 'Nepal',
				'NL' => 'Netherlands',
				'AN' => 'Netherlands Antilles',
				'NC' => 'New Caledonia',
				'NZ' => 'New Zealand',
				'NI' => 'Nicaragua',
				'NE' => 'Niger',
				'NG' => 'Nigeria',
				'NU' => 'Niue',
				'NF' => 'Norfolk Island',
				'MP' => 'Northern Mariana Islands',
				'NO' => 'Norway',
				'OM' => 'Oman',
				'PK' => 'Pakistan',
				'PW' => 'Palau',
				'PS' => 'Palestinian Territory, Occupied',
				'PA' => 'Panama',
				'PG' => 'Papua New Guinea',
				'PY' => 'Paraguay',
				'PE' => 'Peru',
				'PH' => 'Philippines',
				'PN' => 'Pitcairn',
				'PL' => 'Poland',
				'PT' => 'Portugal',
				'PR' => 'Puerto Rico',
				'QA' => 'Qatar',
				'RE' => 'Reunion',
				'RO' => 'Romania',
				'RU' => 'Russian Federation',
				'RW' => 'Rwanda',
				'BL' => 'Saint Barthelemy',
				'SH' => 'Saint Helena',
				'KN' => 'Saint Kitts And Nevis',
				'LC' => 'Saint Lucia',
				'MF' => 'Saint Martin',
				'PM' => 'Saint Pierre And Miquelon',
				'VC' => 'Saint Vincent And Grenadines',
				'WS' => 'Samoa',
				'SM' => 'San Marino',
				'ST' => 'Sao Tome And Principe',
				'SA' => 'Saudi Arabia',
				'SN' => 'Senegal',
				'RS' => 'Serbia',
				'SC' => 'Seychelles',
				'SL' => 'Sierra Leone',
				'SG' => 'Singapore',
				'SK' => 'Slovakia',
				'SI' => 'Slovenia',
				'SB' => 'Solomon Islands',
				'SO' => 'Somalia',
				'ZA' => 'South Africa',
				'GS' => 'South Georgia And Sandwich Isl.',
				'ES' => 'Spain',
				'LK' => 'Sri Lanka',
				'SD' => 'Sudan',
				'SR' => 'Suriname',
				'SJ' => 'Svalbard And Jan Mayen',
				'SZ' => 'Swaziland',
				'SE' => 'Sweden',
				'CH' => 'Switzerland',
				'SY' => 'Syrian Arab Republic',
				'TW' => 'Taiwan',
				'TJ' => 'Tajikistan',
				'TZ' => 'Tanzania',
				'TH' => 'Thailand',
				'TL' => 'Timor-Leste',
				'TG' => 'Togo',
				'TK' => 'Tokelau',
				'TO' => 'Tonga',
				'TT' => 'Trinidad And Tobago',
				'TN' => 'Tunisia',
				'TR' => 'Turkey',
				'TM' => 'Turkmenistan',
				'TC' => 'Turks And Caicos Islands',
				'TV' => 'Tuvalu',
				'UG' => 'Uganda',
				'UA' => 'Ukraine',
				'AE' => 'United Arab Emirates',
				'GB' => 'United Kingdom',
				'US' => 'United States',
				'UM' => 'United States Outlying Islands',
				'UY' => 'Uruguay',
				'UZ' => 'Uzbekistan',
				'VU' => 'Vanuatu',
				'VE' => 'Venezuela',
				'VN' => 'Viet Nam',
				'VG' => 'Virgin Islands, British',
				'VI' => 'Virgin Islands, U.S.',
				'WF' => 'Wallis And Futuna',
				'EH' => 'Western Sahara',
				'YE' => 'Yemen',
				'ZM' => 'Zambia',
				'ZW' => 'Zimbabwe',
			);
			if (array_key_exists($cc, $countries))
			{
				$return = array();
				$return[$cc] = $countries[$cc];
				unset($countries[$cc]);
				return array_merge($return, $countries);
			}
			return $countries;
		}
	}
	
	add_action('widgets_init', create_function('', 'return register_widget("YouBazePreorderWidget");'));
?>